@@ -3,7 +3,11 @@
 pub mod and;
 pub mod apply;
 pub mod atleast;
+/// Avl tree data
+pub mod avl_tree_data;
 pub mod bin_op;
+/// Bit inversion operation on numeric type.
+pub mod bit_inversion;
 pub mod block;
 pub mod bool_to_sigma;
 pub mod byte_array_to_bigint;
@@ -12,6 +16,8 @@ pub mod byte_array_to_long;
 pub mod calc_blake2b256;
 /// Calc Sha256 hash
 pub mod calc_sha256;
+/// Collection.append
+pub mod coll_append;
 /// Get the collection element by index
 pub mod coll_by_index;
 /// Tests whether a predicate holds for at least one element of this collection
@@ -26,6 +32,8 @@ pub mod coll_forall;
 pub mod coll_map;
 /// Collection.size
 pub mod coll_size;
+/// Collection.slice
+pub mod coll_slice;
 /// Collection of elements
 pub mod collection;
 pub mod constant;
@@ -36,9 +44,17 @@ pub mod create_provedlog;
 pub mod decode_point;
 pub mod deserialize_context;
 pub mod deserialize_register;
+/// Numerical downcast
+pub mod downcast;
+/// Exponentiate op for GroupElement
+pub mod exponentiate;
 pub mod expr;
 /// Box value
 pub mod extract_amount;
+/// Box.bytes (serialized box bytes)
+pub mod extract_bytes;
+/// Box.bytesWithoutRef
+pub mod extract_bytes_with_no_ref;
 /// Box.creationInfo (height, tx id + box index)
 pub mod extract_creation_info;
 /// Box id, Blake2b256 hash of this box's content, basically equals to `blake2b256(bytes)`
@@ -51,8 +67,6 @@ pub mod extract_script_bytes;
 pub mod func_value;
 pub mod get_var;
 pub mod global_vars;
-/// Headers data types
-pub mod header;
 /// If-else conditional op
 pub mod if_op;
 /// Logical NOT op
@@ -60,6 +74,8 @@ pub mod logical_not;
 pub mod long_to_byte_array;
 /// Object method call
 pub mod method_call;
+/// Multiply op for GroupElement
+pub mod multiply_group;
 /// Negation operation on numeric type.
 pub mod negation;
 /// Option.get() op
@@ -78,12 +94,18 @@ pub mod sigma_and;
 pub mod sigma_or;
 /// Extract serialized bytes of a SigmaProp value
 pub mod sigma_prop_bytes;
+pub mod subst_const;
 /// Tuple of elements
 pub mod tuple;
 pub mod unary_op;
+/// Numerical upcast
 pub mod upcast;
 /// Variable definition
 pub mod val_def;
 /// Variable reference
 pub mod val_use;
 pub mod value;
+/// Byte-wise XOR op
+pub mod xor;
+/// XOR for collection of booleans
+pub mod xor_of;
