@@ -27,12 +27,6 @@ pub enum ArithOp {
     Max,
     /// Min of two values
     Min,
-    /// Bitwise Or
-    BitOr,
-    /// Bitwise And
-    BitAnd,
-    /// Bitwise Xor
-    BitXor,
 }
 
 impl From<ArithOp> for OpCode {
@@ -44,9 +38,6 @@ impl From<ArithOp> for OpCode {
             ArithOp::Divide => OpCode::DIVISION,
             ArithOp::Max => OpCode::MAX,
             ArithOp::Min => OpCode::MIN,
-            ArithOp::BitOr => OpCode::BIT_OR,
-            ArithOp::BitAnd => OpCode::BIT_AND,
-            ArithOp::BitXor => OpCode::BIT_XOR,
         }
     }
 }
@@ -67,10 +58,6 @@ pub enum RelationOp {
     Le,
     /// Less then
     Lt,
-    /// Logical AND
-    And,
-    /// Logical OR
-    Or,
 }
 
 impl From<RelationOp> for OpCode {
@@ -82,8 +69,50 @@ impl From<RelationOp> for OpCode {
             RelationOp::Gt => OpCode::GT,
             RelationOp::Le => OpCode::LE,
             RelationOp::Lt => OpCode::LT,
-            RelationOp::And => OpCode::BIN_AND,
-            RelationOp::Or => OpCode::BIN_OR,
+        }
+    }
+}
+
+/// Logical operations
+#[derive(PartialEq, Eq, Debug, Clone, Copy)]
+#[cfg_attr(feature = "arbitrary", derive(Arbitrary))]
+pub enum LogicalOp {
+    /// Logical AND
+    And,
+    /// Logical OR
+    Or,
+    /// Logical XOR
+    Xor,
+}
+
+impl From<LogicalOp> for OpCode {
+    fn from(op: LogicalOp) -> Self {
+        match op {
+            LogicalOp::And => OpCode::BIN_AND,
+            LogicalOp::Or => OpCode::BIN_OR,
+            LogicalOp::Xor => OpCode::BIN_XOR,
+        }
+    }
+}
+
+/// Bitwise operations
+#[derive(PartialEq, Eq, Debug, Clone, Copy)]
+#[cfg_attr(feature = "arbitrary", derive(Arbitrary))]
+pub enum BitOp {
+    /// Bitwise Or
+    BitOr,
+    /// Bitwise And
+    BitAnd,
+    /// Bitwise Xor
+    BitXor,
+}
+
+impl From<BitOp> for OpCode {
+    fn from(op: BitOp) -> Self {
+        match op {
+            BitOp::BitOr => OpCode::BIT_OR,
+            BitOp::BitAnd => OpCode::BIT_AND,
+            BitOp::BitXor => OpCode::BIT_XOR,
         }
     }
 }
@@ -96,6 +125,10 @@ pub enum BinOpKind {
     Arith(ArithOp),
     /// Relation operations (equality, comparison, etc.)
     Relation(RelationOp),
+    /// Logical operations
+    Logical(LogicalOp),
+    /// Bitwise operations
+    Bit(BitOp),
 }
 
 impl From<BinOpKind> for OpCode {
@@ -103,6 +136,8 @@ impl From<BinOpKind> for OpCode {
         match op {
             BinOpKind::Arith(o) => o.into(),
             BinOpKind::Relation(o) => o.into(),
+            BinOpKind::Logical(o) => o.into(),
+            BinOpKind::Bit(o) => o.into(),
         }
     }
 }
@@ -124,6 +159,8 @@ impl BinOp {
         match self.kind {
             BinOpKind::Relation(_) => SType::SBoolean,
             BinOpKind::Arith(_) => self.left.tpe(),
+            BinOpKind::Logical(_) => SType::SBoolean,
+            BinOpKind::Bit(_) => self.left.tpe(),
         }
     }
 }
@@ -149,7 +186,10 @@ mod arbitrary {
         fn arbitrary_with(args: Self::Parameters) -> Self::Strategy {
             let numeric_binop = || -> BoxedStrategy<BinOp> {
                 (
-                    any::<ArithOp>().prop_map_into(),
+                    prop_oneof![
+                        any::<ArithOp>().prop_map_into(),
+                        any::<BitOp>().prop_map_into(),
+                    ],
                     any_with::<Expr>(args.clone()),
                     any_with::<Expr>(args.clone()),
                 )
@@ -162,23 +202,36 @@ mod arbitrary {
             };
 
             match args.tpe {
-                SType::SBoolean => (
-                    any::<RelationOp>().prop_map_into(),
-                    any_with::<Expr>(ArbExprParams {
-                        tpe: SType::SAny,
-                        depth: args.depth,
-                    }),
-                    any_with::<Expr>(ArbExprParams {
-                        tpe: SType::SAny,
-                        depth: args.depth,
-                    }),
-                )
-                    .prop_map(|(kind, left, right)| BinOp {
-                        kind,
-                        left: Box::new(left),
-                        right: Box::new(right),
-                    })
-                    .boxed(),
+                SType::SBoolean => prop_oneof![
+                    (
+                        any::<RelationOp>().prop_map_into(),
+                        any_with::<Expr>(ArbExprParams {
+                            tpe: SType::SAny,
+                            depth: args.depth,
+                        }),
+                        any_with::<Expr>(ArbExprParams {
+                            tpe: SType::SAny,
+                            depth: args.depth,
+                        }),
+                    ),
+                    (
+                        any::<LogicalOp>().prop_map_into(),
+                        any_with::<Expr>(ArbExprParams {
+                            tpe: SType::SBoolean,
+                            depth: args.depth,
+                        }),
+                        any_with::<Expr>(ArbExprParams {
+                            tpe: SType::SBoolean,
+                            depth: args.depth,
+                        }),
+                    )
+                ]
+                .prop_map(|(kind, left, right)| BinOp {
+                    kind,
+                    left: Box::new(left),
+                    right: Box::new(right),
+                })
+                .boxed(),
 
                 SType::SByte => numeric_binop(),
                 SType::SShort => numeric_binop(),
@@ -209,12 +262,13 @@ mod arbitrary {
 }
 
 #[cfg(test)]
+#[allow(clippy::panic)]
 mod tests {
 
     use super::*;
     use crate::mir::constant::Constant;
+    use crate::mir::constant::Literal::Boolean;
     use crate::mir::expr::Expr;
-    use crate::mir::value::Value::Boolean;
     use crate::serialization::sigma_serialize_roundtrip;
     use crate::serialization::SigmaSerializable;
     use proptest::prelude::*;
@@ -236,7 +290,7 @@ mod tests {
         assert_eq!(
             e,
             Ok(Expr::BinOp(BinOp {
-                kind: BinOpKind::Relation(RelationOp::And,),
+                kind: BinOpKind::Logical(LogicalOp::And,),
                 left: Expr::Const(Constant {
                     tpe: SType::SBoolean,
                     v: Boolean(true),
