@@ -3,19 +3,19 @@
 use std::convert::TryFrom;
 use std::convert::TryInto;
 
-use crate::has_opcode::HasOpCode;
-use crate::serialization::op_code::OpCode;
 use crate::types::stype::LiftIntoSType;
 use crate::types::stype::SType;
 
 use super::and::And;
 use super::apply::Apply;
 use super::bin_op::BinOp;
+use super::bit_inversion::BitInversion;
 use super::block::BlockValue;
 use super::bool_to_sigma::BoolToSigmaProp;
 use super::byte_array_to_long::ByteArrayToLong;
 use super::calc_blake2b256::CalcBlake2b256;
 use super::calc_sha256::CalcSha256;
+use super::coll_append::Append;
 use super::coll_by_index::ByIndex;
 use super::coll_exists::Exists;
 use super::coll_filter::Filter;
@@ -23,14 +23,19 @@ use super::coll_fold::Fold;
 use super::coll_forall::ForAll;
 use super::coll_map::Map;
 use super::coll_size::SizeOf;
+use super::coll_slice::Slice;
 use super::collection::Collection;
 use super::constant::Constant;
 use super::constant::ConstantPlaceholder;
+use super::constant::Literal;
 use super::constant::TryExtractFrom;
 use super::constant::TryExtractFromError;
 use super::create_provedlog::CreateProveDlog;
 use super::decode_point::DecodePoint;
+use super::exponentiate::Exponentiate;
 use super::extract_amount::ExtractAmount;
+use super::extract_bytes::ExtractBytes;
+use super::extract_bytes_with_no_ref::ExtractBytesWithNoRef;
 use super::extract_creation_info::ExtractCreationInfo;
 use super::extract_id::ExtractId;
 use super::extract_reg_as::ExtractRegisterAs;
@@ -41,6 +46,7 @@ use super::if_op::If;
 use super::logical_not::LogicalNot;
 use super::long_to_byte_array::LongToByteArray;
 use super::method_call::MethodCall;
+use super::multiply_group::MultiplyGroup;
 use super::negation::Negation;
 use super::option_get::OptionGet;
 use super::option_get_or_else::OptionGetOrElse;
@@ -51,11 +57,12 @@ use super::select_field::SelectField;
 use super::sigma_and::SigmaAnd;
 use super::sigma_or::SigmaOr;
 use super::sigma_prop_bytes::SigmaPropBytes;
+use super::subst_const::SubstConstants;
 use super::tuple::Tuple;
 use super::upcast::Upcast;
 use super::val_def::ValDef;
 use super::val_use::ValUse;
-use super::value::Value;
+use super::xor::Xor;
 
 extern crate derive_more;
 use crate::mir::atleast::Atleast;
@@ -63,18 +70,25 @@ use crate::mir::byte_array_to_bigint::ByteArrayToBigInt;
 use crate::mir::create_prove_dh_tuple::CreateProveDhTuple;
 use crate::mir::deserialize_context::DeserializeContext;
 use crate::mir::deserialize_register::DeserializeRegister;
+use crate::mir::downcast::Downcast;
 use crate::mir::get_var::GetVar;
+use crate::mir::xor_of::XorOf;
 use bounded_vec::BoundedVecOutOfBounds;
 use derive_more::From;
 use derive_more::TryInto;
+use thiserror::Error;
 
 #[derive(PartialEq, Eq, Debug, Clone, From, TryInto)]
 /// Expression in ErgoTree
 pub enum Expr {
+    /// Append - Concatenation of two collections
+    Append(Append),
     /// Constant value
     Const(Constant),
     /// Placeholder for a constant
     ConstPlaceholder(ConstantPlaceholder),
+    /// Substitute constants in serialized ergo tree
+    SubstConstants(SubstConstants),
     /// Convert byte array to SLong
     ByteArrayToLong(ByteArrayToLong),
     /// Convert byte array to SLong
@@ -118,12 +132,16 @@ pub enum Expr {
     And(And),
     /// Logical OR
     Or(Or),
+    /// Byte-wise XOR
+    Xor(Xor),
     /// THRESHOLD composition for sigma expressions
     Atleast(Atleast),
     /// LogicalNot
     LogicalNot(LogicalNot),
     /// Negation on numeric type
     Negation(Negation),
+    /// Bit inversion on numeric type
+    BitInversion(BitInversion),
     /// Option.get method
     OptionGet(OptionGet),
     /// Option.isDefined method
@@ -134,6 +152,10 @@ pub enum Expr {
     ExtractAmount(ExtractAmount),
     /// Extract register's value (box.RX properties)
     ExtractRegisterAs(ExtractRegisterAs),
+    /// Extract serialized box bytes
+    ExtractBytes(ExtractBytes),
+    /// Extract serialized box bytes excluding transaction_id & index
+    ExtractBytesWithNoRef(ExtractBytesWithNoRef),
     /// Extract box's guarding script serialized to bytes
     ExtractScriptBytes(ExtractScriptBytes),
     /// Tuple of height when block got included into the blockchain and transaction identifier with
@@ -145,6 +167,8 @@ pub enum Expr {
     ByIndex(ByIndex),
     /// Collection size
     SizeOf(SizeOf),
+    /// Collection slice
+    Slice(Slice),
     /// Collection fold op
     Fold(Fold),
     /// Collection map op
@@ -161,6 +185,8 @@ pub enum Expr {
     BoolToSigmaProp(BoolToSigmaProp),
     /// Upcast numeric value
     Upcast(Upcast),
+    /// Downcast numeric value
+    Downcast(Downcast),
     /// Create proveDlog from GroupElement(PK)
     CreateProveDlog(CreateProveDlog),
     /// Create proveDlog from GroupElement(PK)
@@ -183,73 +209,22 @@ pub enum Expr {
     /// available as `getVar[Coll[Byte]](id)` On evaluation returns the result of the
     /// script execution in the current context
     DeserializeContext(DeserializeContext),
+    /// MultiplyGroup op for GroupElement
+    MultiplyGroup(MultiplyGroup),
+    /// Exponentiate op for GroupElement
+    Exponentiate(Exponentiate),
+    /// XOR for collection of booleans
+    XorOf(XorOf),
 }
 
 impl Expr {
-    /// Code (used in serialization)
-    pub(crate) fn op_code(&self) -> OpCode {
-        match self {
-            Expr::Const(_) => panic!("constant does not have op code assigned"),
-            Expr::ConstPlaceholder(op) => op.op_code(),
-            Expr::ByteArrayToLong(op) => op.op_code(),
-            Expr::ByteArrayToBigInt(op) => op.op_code(),
-            Expr::LongToByteArray(op) => op.op_code(),
-            Expr::Collection(op) => op.op_code(),
-            Expr::GlobalVars(op) => op.op_code(),
-            Expr::MethodCall(op) => op.op_code(),
-            Expr::ProperyCall(op) => op.op_code(),
-            Expr::Global => OpCode::GLOBAL,
-            Expr::Context => OpCode::CONTEXT,
-            Expr::OptionGet(op) => op.op_code(),
-            Expr::ExtractRegisterAs(op) => op.op_code(),
-            Expr::BinOp(op) => op.op_code(),
-            Expr::BlockValue(op) => op.op_code(),
-            Expr::ValUse(op) => op.op_code(),
-            Expr::FuncValue(op) => op.op_code(),
-            Expr::Apply(op) => op.op_code(),
-            Expr::ValDef(op) => op.op_code(),
-            Expr::ExtractAmount(op) => op.op_code(),
-            Expr::SelectField(op) => op.op_code(),
-            Expr::Fold(op) => op.op_code(),
-            Expr::CalcBlake2b256(op) => op.op_code(),
-            Expr::CalcSha256(op) => op.op_code(),
-            Expr::And(op) => op.op_code(),
-            Expr::Or(op) => op.op_code(),
-            Expr::Atleast(op) => op.op_code(),
-            Expr::LogicalNot(op) => op.op_code(),
-            Expr::Map(op) => op.op_code(),
-            Expr::Filter(op) => op.op_code(),
-            Expr::BoolToSigmaProp(op) => op.op_code(),
-            Expr::Upcast(op) => op.op_code(),
-            Expr::If(op) => op.op_code(),
-            Expr::ByIndex(op) => op.op_code(),
-            Expr::ExtractScriptBytes(op) => op.op_code(),
-            Expr::SizeOf(op) => op.op_code(),
-            Expr::CreateProveDlog(op) => op.op_code(),
-            Expr::CreateProveDhTuple(op) => op.op_code(),
-            Expr::ExtractCreationInfo(op) => op.op_code(),
-            Expr::Exists(op) => op.op_code(),
-            Expr::ExtractId(op) => op.op_code(),
-            Expr::SigmaPropBytes(op) => op.op_code(),
-            Expr::OptionIsDefined(op) => op.op_code(),
-            Expr::OptionGetOrElse(op) => op.op_code(),
-            Expr::Negation(op) => op.op_code(),
-            Expr::ForAll(op) => op.op_code(),
-            Expr::Tuple(op) => op.op_code(),
-            Expr::DecodePoint(op) => op.op_code(),
-            Expr::SigmaAnd(op) => op.op_code(),
-            Expr::SigmaOr(op) => op.op_code(),
-            Expr::GetVar(op) => op.op_code(),
-            Expr::DeserializeRegister(op) => op.op_code(),
-            Expr::DeserializeContext(op) => op.op_code(),
-        }
-    }
-
     /// Type of the expression
     pub fn tpe(&self) -> SType {
         match self {
+            Expr::Append(ap) => ap.tpe(),
             Expr::Const(v) => v.tpe.clone(),
             Expr::Collection(v) => v.tpe(),
+            Expr::SubstConstants(v) => v.tpe(),
             Expr::ByteArrayToLong(v) => v.tpe(),
             Expr::ByteArrayToBigInt(v) => v.tpe(),
             Expr::LongToByteArray(v) => v.tpe(),
@@ -274,16 +249,19 @@ impl Expr {
             Expr::ExtractAmount(v) => v.tpe(),
             Expr::And(v) => v.tpe(),
             Expr::Or(v) => v.tpe(),
+            Expr::Xor(v) => v.tpe(),
             Expr::Atleast(v) => v.tpe(),
             Expr::LogicalNot(v) => v.tpe(),
             Expr::Map(v) => v.tpe(),
             Expr::Filter(v) => v.tpe(),
             Expr::BoolToSigmaProp(v) => v.tpe(),
             Expr::Upcast(v) => v.tpe(),
+            Expr::Downcast(v) => v.tpe(),
             Expr::If(v) => v.tpe(),
             Expr::ByIndex(v) => v.tpe(),
             Expr::ExtractScriptBytes(v) => v.tpe(),
             Expr::SizeOf(v) => v.tpe(),
+            Expr::Slice(v) => v.tpe(),
             Expr::CreateProveDlog(v) => v.tpe(),
             Expr::CreateProveDhTuple(v) => v.tpe(),
             Expr::ExtractCreationInfo(v) => v.tpe(),
@@ -293,6 +271,7 @@ impl Expr {
             Expr::OptionIsDefined(v) => v.tpe(),
             Expr::OptionGetOrElse(v) => v.tpe(),
             Expr::Negation(v) => v.tpe(),
+            Expr::BitInversion(v) => v.tpe(),
             Expr::ForAll(v) => v.tpe(),
             Expr::Tuple(v) => v.tpe(),
             Expr::DecodePoint(v) => v.tpe(),
@@ -301,6 +280,11 @@ impl Expr {
             Expr::DeserializeRegister(v) => v.tpe(),
             Expr::DeserializeContext(v) => v.tpe(),
             Expr::GetVar(v) => v.tpe(),
+            Expr::MultiplyGroup(v) => v.tpe(),
+            Expr::Exponentiate(v) => v.tpe(),
+            Expr::XorOf(v) => v.tpe(),
+            Expr::ExtractBytes(v) => v.tpe(),
+            Expr::ExtractBytesWithNoRef(v) => v.tpe(),
         }
     }
 
@@ -313,9 +297,12 @@ impl Expr {
     }
 
     /// Check if given expected_tpe type is the same as the expression's post-evaluation type
-    pub fn check_post_eval_tpe(&self, expected_tpe: SType) -> Result<(), InvalidExprEvalTypeError> {
+    pub fn check_post_eval_tpe(
+        &self,
+        expected_tpe: &SType,
+    ) -> Result<(), InvalidExprEvalTypeError> {
         let expr_tpe = self.post_eval_tpe();
-        if expr_tpe == expected_tpe {
+        if &expr_tpe == expected_tpe {
             Ok(())
         } else {
             Err(InvalidExprEvalTypeError(format!(
@@ -332,7 +319,7 @@ impl Expr {
     }
 }
 
-impl<T: Into<Value> + LiftIntoSType> From<T> for Expr {
+impl<T: Into<Literal> + LiftIntoSType> From<T> for Expr {
     fn from(t: T) -> Self {
         Expr::Const(Constant {
             tpe: T::stype(),
@@ -342,7 +329,8 @@ impl<T: Into<Value> + LiftIntoSType> From<T> for Expr {
 }
 
 /// Unexpected argument on node construction (i.e non-Option input in OptionGet)
-#[derive(PartialEq, Eq, Debug, Clone)]
+#[derive(Error, PartialEq, Eq, Debug, Clone)]
+#[error("InvalidArgumentError: {0}")]
 pub struct InvalidArgumentError(pub String);
 
 /// Invalid (unexpected) expr type
@@ -376,6 +364,8 @@ impl<T: TryFrom<Expr>> TryExtractFrom<Expr> for T {
 
 #[cfg(feature = "arbitrary")]
 #[allow(clippy::unwrap_used)]
+#[allow(clippy::panic)]
+#[allow(clippy::todo)]
 /// Arbitrary impl
 pub(crate) mod arbitrary {
     use super::*;
@@ -519,10 +509,19 @@ pub(crate) mod arbitrary {
 
     fn coll_non_nested_expr(elem_tpe: &SType) -> BoxedStrategy<Expr> {
         match elem_tpe {
+            SType::SBoolean => any_with::<Constant>(SType::SColl(Box::new(SType::SBoolean)).into())
+                .prop_map(Expr::Const)
+                .boxed(),
             SType::SByte => any_with::<Constant>(SType::SColl(Box::new(SType::SByte)).into())
                 .prop_map(Expr::Const)
                 .boxed(),
-            SType::SBoolean => any_with::<Constant>(SType::SColl(Box::new(SType::SBoolean)).into())
+            SType::SShort => any_with::<Constant>(SType::SColl(Box::new(SType::SShort)).into())
+                .prop_map(Expr::Const)
+                .boxed(),
+            SType::SInt => any_with::<Constant>(SType::SColl(Box::new(SType::SInt)).into())
+                .prop_map(Expr::Const)
+                .boxed(),
+            SType::SLong => any_with::<Constant>(SType::SColl(Box::new(SType::SLong)).into())
                 .prop_map(Expr::Const)
                 .boxed(),
             _ => todo!("Collection of {0:?} is not yet implemented", elem_tpe),
