@@ -1,22 +1,23 @@
 //! Ergo data type
 
-use std::convert::TryFrom;
 use std::convert::TryInto;
+use std::rc::Rc;
 
 use impl_trait_for_tuples::impl_for_tuples;
-use num_bigint::BigInt;
 
-use crate::ir_ergo_box::IrBoxId;
+use crate::bigint256::BigInt256;
+use crate::chain::ergo_box::ErgoBox;
+use crate::chain::header::Header;
+use crate::chain::preheader::PreHeader;
 use crate::sigma_protocol::dlog_group::EcPoint;
-use crate::sigma_protocol::sigma_boolean::ProveDlog;
-use crate::sigma_protocol::sigma_boolean::SigmaBoolean;
-use crate::sigma_protocol::sigma_boolean::SigmaProofOfKnowledgeTree;
 use crate::sigma_protocol::sigma_boolean::SigmaProp;
 use crate::types::stuple::TupleItems;
 use crate::types::stype::LiftIntoSType;
 use crate::types::stype::SType;
 use crate::util::AsVecI8;
 
+use super::avl_tree_data::AvlTreeData;
+use super::constant::Literal;
 use super::constant::TryExtractFrom;
 use super::constant::TryExtractFromError;
 use super::constant::TryExtractInto;
@@ -44,7 +45,7 @@ impl NativeColl {
 
 /// Collection elements
 #[derive(PartialEq, Eq, Debug, Clone)]
-pub enum CollKind {
+pub enum CollKind<T> {
     /// Collection elements stored as a vector of Rust values
     NativeColl(NativeColl),
     /// Collection elements stored as a vector of Value's
@@ -52,13 +53,20 @@ pub enum CollKind {
         /// Collection element type
         elem_tpe: SType,
         /// Collection elements
-        items: Vec<Value>,
+        items: Vec<T>,
     },
 }
 
-impl CollKind {
+impl<T> CollKind<T>
+where
+    T: PartialEq + Eq + Clone,
+    T: From<i8>,
+    i8: TryExtractFrom<T>,
+    Vec<i8>: TryExtractFrom<T>,
+    Vec<T>: TryExtractFrom<T>,
+{
     /// Build a collection from items, storing them as Rust types values when neccessary
-    pub fn from_vec(elem_tpe: SType, items: Vec<Value>) -> Result<CollKind, TryExtractFromError> {
+    pub fn from_vec(elem_tpe: SType, items: Vec<T>) -> Result<CollKind<T>, TryExtractFromError> {
         match elem_tpe {
             SType::SByte => items
                 .into_iter()
@@ -72,8 +80,8 @@ impl CollKind {
     /// Build a collection from items where each is a collection as well, storing them as Rust types values when neccessary
     pub fn from_vec_vec(
         elem_tpe: SType,
-        items: Vec<Value>,
-    ) -> Result<CollKind, TryExtractFromError> {
+        items: Vec<T>,
+    ) -> Result<CollKind<T>, TryExtractFromError> {
         match elem_tpe {
             SType::SByte => items
                 .into_iter()
@@ -82,7 +90,7 @@ impl CollKind {
                 .map(|bytes| CollKind::NativeColl(NativeColl::CollByte(bytes.concat()))),
             _ => items
                 .into_iter()
-                .map(|v| v.try_extract_into::<Vec<Value>>())
+                .map(|v| v.try_extract_into::<Vec<T>>())
                 .collect::<Result<Vec<_>, _>>()
                 .map(|v| CollKind::WrappedColl {
                     elem_tpe,
@@ -100,7 +108,7 @@ impl CollKind {
     }
 
     /// Return items, as vector of Values
-    pub fn as_vec(&self) -> Vec<Value> {
+    pub fn as_vec(&self) -> Vec<T> {
         match self {
             CollKind::NativeColl(NativeColl::CollByte(coll_byte)) => coll_byte
                 .clone()
@@ -138,21 +146,25 @@ pub enum Value {
     /// Long
     Long(i64),
     /// Big integer
-    BigInt(BigInt),
+    BigInt(BigInt256),
     /// GroupElement
     GroupElement(Box<EcPoint>),
     /// Sigma property
     SigmaProp(Box<SigmaProp>),
-    /// Box
-    CBox(IrBoxId),
+    /// Ergo box
+    CBox(Rc<ErgoBox>),
     /// AVL tree
-    AvlTree,
+    AvlTree(Box<AvlTreeData>),
     /// Collection of values of the same type
-    Coll(CollKind),
+    Coll(CollKind<Value>),
     /// Tuple (arbitrary type values)
     Tup(TupleItems<Value>),
     /// Transaction(and blockchain) context info
     Context,
+    /// Block header
+    Header(Box<Header>),
+    /// Header with predictable data
+    PreHeader(Box<PreHeader>),
     /// Global which is used to define global methods
     Global,
     /// Optional value
@@ -198,6 +210,35 @@ impl<T: Into<Value>> From<Option<T>> for Value {
     }
 }
 
+impl From<Literal> for Value {
+    fn from(lit: Literal) -> Self {
+        match lit {
+            Literal::Boolean(b) => Value::Boolean(b),
+            Literal::Byte(b) => Value::Byte(b),
+            Literal::Short(s) => Value::Short(s),
+            Literal::Int(i) => Value::Int(i),
+            Literal::Long(l) => Value::Long(l),
+            Literal::BigInt(b) => Value::BigInt(b),
+            Literal::SigmaProp(s) => Value::SigmaProp(s),
+            Literal::GroupElement(e) => Value::GroupElement(e),
+            Literal::CBox(b) => Value::CBox(b),
+            Literal::Coll(coll) => {
+                let converted_coll = match coll {
+                    CollKind::NativeColl(n) => CollKind::NativeColl(n),
+                    CollKind::WrappedColl { elem_tpe, items } => CollKind::WrappedColl {
+                        elem_tpe,
+                        items: items.into_iter().map(Value::from).collect(),
+                    },
+                };
+                Value::Coll(converted_coll)
+            }
+            Literal::AvlTree(a) => Value::AvlTree(a),
+            Literal::Opt(lit) => Value::Opt(Box::new(lit.into_iter().next().map(Value::from))),
+            Literal::Tup(t) => Value::Tup(t.mapped(Value::from)),
+        }
+    }
+}
+
 /// Marker trait to select types which stored as Vec of wrapped Value's
 pub trait StoreWrapped {}
 
@@ -205,8 +246,9 @@ impl StoreWrapped for bool {}
 impl StoreWrapped for i16 {}
 impl StoreWrapped for i32 {}
 impl StoreWrapped for i64 {}
-impl StoreWrapped for BigInt {}
-impl StoreWrapped for IrBoxId {}
+impl StoreWrapped for BigInt256 {}
+impl StoreWrapped for Header {}
+impl StoreWrapped for Rc<ErgoBox> {}
 impl StoreWrapped for EcPoint {}
 impl StoreWrapped for SigmaProp {}
 impl<T: StoreWrapped> StoreWrapped for Option<T> {}
@@ -225,7 +267,7 @@ impl<T: LiftIntoSType + StoreWrapped + Into<Value>> From<Vec<T>> for Value {
     }
 }
 
-#[allow(clippy::clippy::from_over_into)]
+#[allow(clippy::from_over_into)]
 #[allow(clippy::unwrap_used)]
 #[impl_for_tuples(2, 4)]
 impl Into<Value> for Tuple {
@@ -235,6 +277,15 @@ impl Into<Value> for Tuple {
     }
 }
 
+impl From<Vec<Rc<ErgoBox>>> for Value {
+    fn from(v: Vec<Rc<ErgoBox>>) -> Self {
+        Value::Coll(CollKind::WrappedColl {
+            elem_tpe: SType::SBox,
+            items: v.into_iter().map(|i| i.into()).collect(),
+        })
+    }
+}
+
 impl TryExtractFrom<Value> for bool {
     fn try_extract_from(cv: Value) -> Result<bool, TryExtractFromError> {
         match cv {
@@ -307,12 +358,36 @@ impl TryExtractFrom<Value> for SigmaProp {
     }
 }
 
-impl TryExtractFrom<Value> for IrBoxId {
+impl TryExtractFrom<Value> for Rc<ErgoBox> {
     fn try_extract_from(c: Value) -> Result<Self, TryExtractFromError> {
         match c {
             Value::CBox(b) => Ok(b),
             _ => Err(TryExtractFromError(format!(
-                "expected IrErgoBox, found {:?}",
+                "expected ErgoBox, found {:?}",
+                c
+            ))),
+        }
+    }
+}
+
+impl TryExtractFrom<Value> for Header {
+    fn try_extract_from(c: Value) -> Result<Self, TryExtractFromError> {
+        match c {
+            Value::Header(h) => Ok(*h),
+            _ => Err(TryExtractFromError(format!(
+                "expected Header, found {:?}",
+                c
+            ))),
+        }
+    }
+}
+
+impl TryExtractFrom<Value> for PreHeader {
+    fn try_extract_from(c: Value) -> Result<Self, TryExtractFromError> {
+        match c {
+            Value::PreHeader(ph) => Ok(*ph),
+            _ => Err(TryExtractFromError(format!(
+                "expected PreHeader, found {:?}",
                 c
             ))),
         }
@@ -342,6 +417,36 @@ impl<T: TryExtractFrom<Value> + StoreWrapped> TryExtractFrom<Value> for Vec<T> {
     }
 }
 
+impl<T: TryExtractFrom<Value> + StoreWrapped, const N: usize> TryExtractFrom<Value> for [T; N] {
+    fn try_extract_from(c: Value) -> Result<Self, TryExtractFromError> {
+        match c {
+            Value::Coll(coll) => match coll {
+                CollKind::WrappedColl {
+                    elem_tpe: _,
+                    items: v,
+                } => {
+                    let v = v
+                        .into_iter()
+                        .map(T::try_extract_from)
+                        .collect::<Result<Vec<_>, _>>()?;
+                    let len = v.len();
+                    v.try_into().map_err(|_| TryExtractFromError(format!("can't convert vec of {:?} with length of {:?} to array with length of {:?}", std::any::type_name::<T>(), len, N)))
+                }
+                _ => Err(TryExtractFromError(format!(
+                    "expected {:?}, found {:?}",
+                    std::any::type_name::<Self>(),
+                    coll
+                ))),
+            },
+            _ => Err(TryExtractFromError(format!(
+                "expected {:?}, found {:?}",
+                std::any::type_name::<Self>(),
+                c
+            ))),
+        }
+    }
+}
+
 impl TryExtractFrom<Value> for Vec<i8> {
     fn try_extract_from(v: Value) -> Result<Self, TryExtractFromError> {
         match v {
@@ -375,7 +480,7 @@ impl TryExtractFrom<Value> for Value {
     }
 }
 
-impl TryExtractFrom<Value> for BigInt {
+impl TryExtractFrom<Value> for BigInt256 {
     fn try_extract_from(v: Value) -> Result<Self, TryExtractFromError> {
         match v {
             Value::BigInt(bi) => Ok(bi),
@@ -388,30 +493,22 @@ impl TryExtractFrom<Value> for BigInt {
     }
 }
 
-impl<T: TryExtractFrom<Value> + StoreWrapped> TryExtractFrom<Vec<Value>> for Vec<T> {
-    fn try_extract_from(v: Vec<Value>) -> Result<Self, TryExtractFromError> {
-        v.into_iter().map(|it| it.try_extract_into::<T>()).collect()
+impl TryExtractFrom<Value> for AvlTreeData {
+    fn try_extract_from(v: Value) -> Result<Self, TryExtractFromError> {
+        match v {
+            Value::AvlTree(a) => Ok(*a),
+            _ => Err(TryExtractFromError(format!(
+                "expected {:?}, found {:?}",
+                std::any::type_name::<Self>(),
+                v
+            ))),
+        }
     }
 }
 
-impl TryFrom<Value> for ProveDlog {
-    type Error = TryExtractFromError;
-    fn try_from(cv: Value) -> Result<Self, Self::Error> {
-        match cv {
-            Value::SigmaProp(sp) => match sp.value() {
-                SigmaBoolean::ProofOfKnowledge(SigmaProofOfKnowledgeTree::ProveDlog(
-                    prove_dlog,
-                )) => Ok(prove_dlog.clone()),
-                _ => Err(TryExtractFromError(format!(
-                    "expected ProveDlog, found {:?}",
-                    sp
-                ))),
-            },
-            _ => Err(TryExtractFromError(format!(
-                "expected SigmaProp, found {:?}",
-                cv
-            ))),
-        }
+impl<T: TryExtractFrom<Value> + StoreWrapped> TryExtractFrom<Vec<Value>> for Vec<T> {
+    fn try_extract_from(v: Vec<Value>) -> Result<Self, TryExtractFromError> {
+        v.into_iter().map(|it| it.try_extract_into::<T>()).collect()
     }
 }
 
@@ -432,7 +529,7 @@ impl<T: TryExtractFrom<Value>> TryExtractFrom<Value> for Option<T> {
         match v {
             Value::Opt(opt) => opt.map(T::try_extract_from).transpose(),
             _ => Err(TryExtractFromError(format!(
-                "expected Context, found {:?}",
+                "expected Option, found {:?}",
                 v
             ))),
         }
