@@ -7,13 +7,17 @@ use crate::mir::and::And;
 use crate::mir::apply::Apply;
 use crate::mir::atleast::Atleast;
 use crate::mir::bin_op::ArithOp;
+use crate::mir::bin_op::BitOp;
+use crate::mir::bin_op::LogicalOp;
 use crate::mir::bin_op::RelationOp;
+use crate::mir::bit_inversion::BitInversion;
 use crate::mir::block::BlockValue;
 use crate::mir::bool_to_sigma::BoolToSigmaProp;
 use crate::mir::byte_array_to_bigint::ByteArrayToBigInt;
 use crate::mir::byte_array_to_long::ByteArrayToLong;
 use crate::mir::calc_blake2b256::CalcBlake2b256;
 use crate::mir::calc_sha256::CalcSha256;
+use crate::mir::coll_append::Append;
 use crate::mir::coll_by_index::ByIndex;
 use crate::mir::coll_exists::Exists;
 use crate::mir::coll_filter::Filter;
@@ -21,6 +25,7 @@ use crate::mir::coll_fold::Fold;
 use crate::mir::coll_forall::ForAll;
 use crate::mir::coll_map::Map;
 use crate::mir::coll_size::SizeOf;
+use crate::mir::coll_slice::Slice;
 use crate::mir::collection::bool_const_coll_sigma_parse;
 use crate::mir::collection::coll_sigma_parse;
 use crate::mir::collection::coll_sigma_serialize;
@@ -31,8 +36,12 @@ use crate::mir::create_provedlog::CreateProveDlog;
 use crate::mir::decode_point::DecodePoint;
 use crate::mir::deserialize_context::DeserializeContext;
 use crate::mir::deserialize_register::DeserializeRegister;
+use crate::mir::downcast::Downcast;
+use crate::mir::exponentiate::Exponentiate;
 use crate::mir::expr::Expr;
 use crate::mir::extract_amount::ExtractAmount;
+use crate::mir::extract_bytes::ExtractBytes;
+use crate::mir::extract_bytes_with_no_ref::ExtractBytesWithNoRef;
 use crate::mir::extract_creation_info::ExtractCreationInfo;
 use crate::mir::extract_id::ExtractId;
 use crate::mir::extract_reg_as::ExtractRegisterAs;
@@ -44,6 +53,7 @@ use crate::mir::if_op::If;
 use crate::mir::logical_not::LogicalNot;
 use crate::mir::long_to_byte_array::LongToByteArray;
 use crate::mir::method_call::MethodCall;
+use crate::mir::multiply_group::MultiplyGroup;
 use crate::mir::negation::Negation;
 use crate::mir::option_get::OptionGet;
 use crate::mir::option_get_or_else::OptionGetOrElse;
@@ -54,24 +64,24 @@ use crate::mir::select_field::SelectField;
 use crate::mir::sigma_and::SigmaAnd;
 use crate::mir::sigma_or::SigmaOr;
 use crate::mir::sigma_prop_bytes::SigmaPropBytes;
+use crate::mir::subst_const::SubstConstants;
 use crate::mir::tuple::Tuple;
 use crate::mir::upcast::Upcast;
 use crate::mir::val_def::ValDef;
 use crate::mir::val_use::ValUse;
+use crate::mir::xor::Xor;
+use crate::serialization::SigmaSerializeResult;
 use crate::serialization::{
-    sigma_byte_reader::SigmaByteRead, SerializationError, SigmaSerializable,
+    sigma_byte_reader::SigmaByteRead, SigmaParsingError, SigmaSerializable,
 };
 
+use crate::mir::xor_of::XorOf;
 use crate::serialization::types::TypeCode;
-use std::io;
 
 impl Expr {
     /// Parse expression from byte stream. This function should be used instead of
     /// `sigma_parse` when tag byte is already read for look-ahead
-    pub fn parse_with_tag<R: SigmaByteRead>(
-        r: &mut R,
-        tag: u8,
-    ) -> Result<Self, SerializationError> {
+    pub fn parse_with_tag<R: SigmaByteRead>(r: &mut R, tag: u8) -> Result<Self, SigmaParsingError> {
         let res = if tag <= OpCode::LAST_CONSTANT_CODE.value() {
             let t_code = TypeCode::parse(tag)?;
             let constant = Constant::parse_with_type_code(r, t_code)?;
@@ -79,6 +89,7 @@ impl Expr {
         } else {
             let op_code = OpCode::parse(tag);
             match op_code {
+                OpCode::APPEND => Ok(Append::sigma_parse(r)?.into()),
                 OpCode::FOLD => Ok(Fold::sigma_parse(r)?.into()),
                 ConstantPlaceholder::OP_CODE => {
                     let cp = ConstantPlaceholder::sigma_parse(r)?;
@@ -96,6 +107,7 @@ impl Expr {
                 OpCode::INPUTS => Ok(Expr::GlobalVars(GlobalVars::Inputs)),
                 OpCode::OUTPUTS => Ok(Expr::GlobalVars(GlobalVars::Outputs)),
                 OpCode::MINER_PUBKEY => Ok(Expr::GlobalVars(GlobalVars::MinerPubKey)),
+                OpCode::GROUP_GENERATOR => Ok(Expr::GlobalVars(GlobalVars::GroupGenerator)),
                 OpCode::GLOBAL => Ok(Expr::Global),
                 OpCode::PROPERTY_CALL => Ok(Expr::ProperyCall(PropertyCall::sigma_parse(r)?)),
                 OpCode::METHOD_CALL => Ok(Expr::MethodCall(MethodCall::sigma_parse(r)?)),
@@ -105,14 +117,18 @@ impl Expr {
                 OptionGetOrElse::OP_CODE => Ok(OptionGetOrElse::sigma_parse(r)?.into()),
                 ExtractRegisterAs::OP_CODE => Ok(ExtractRegisterAs::sigma_parse(r)?.into()),
                 ExtractScriptBytes::OP_CODE => Ok(ExtractScriptBytes::sigma_parse(r)?.into()),
+                ExtractBytes::OP_CODE => Ok(ExtractBytes::sigma_parse(r)?.into()),
+                ExtractBytesWithNoRef::OP_CODE => Ok(ExtractBytesWithNoRef::sigma_parse(r)?.into()),
                 ExtractCreationInfo::OP_CODE => Ok(ExtractCreationInfo::sigma_parse(r)?.into()),
                 ExtractId::OP_CODE => Ok(ExtractId::sigma_parse(r)?.into()),
                 OpCode::EQ => Ok(bin_op_sigma_parse(RelationOp::Eq.into(), r)?),
                 OpCode::NEQ => Ok(bin_op_sigma_parse(RelationOp::NEq.into(), r)?),
                 Negation::OP_CODE => Ok(Negation::sigma_parse(r)?.into()),
+                BitInversion::OP_CODE => Ok(BitInversion::sigma_parse(r)?.into()),
                 OpCode::LOGICAL_NOT => Ok(LogicalNot::sigma_parse(r)?.into()),
-                OpCode::BIN_AND => Ok(bin_op_sigma_parse(RelationOp::And.into(), r)?),
-                OpCode::BIN_OR => Ok(bin_op_sigma_parse(RelationOp::Or.into(), r)?),
+                OpCode::BIN_AND => Ok(bin_op_sigma_parse(LogicalOp::And.into(), r)?),
+                OpCode::BIN_OR => Ok(bin_op_sigma_parse(LogicalOp::Or.into(), r)?),
+                OpCode::BIN_XOR => Ok(bin_op_sigma_parse(LogicalOp::Xor.into(), r)?),
                 OpCode::GT => Ok(bin_op_sigma_parse(RelationOp::Gt.into(), r)?),
                 OpCode::LT => Ok(bin_op_sigma_parse(RelationOp::Lt.into(), r)?),
                 OpCode::GE => Ok(bin_op_sigma_parse(RelationOp::Ge.into(), r)?),
@@ -123,9 +139,9 @@ impl Expr {
                 OpCode::DIVISION => Ok(bin_op_sigma_parse(ArithOp::Divide.into(), r)?),
                 OpCode::MAX => Ok(bin_op_sigma_parse(ArithOp::Max.into(), r)?),
                 OpCode::MIN => Ok(bin_op_sigma_parse(ArithOp::Min.into(), r)?),
-                OpCode::BIT_OR => Ok(bin_op_sigma_parse(ArithOp::BitOr.into(), r)?),
-                OpCode::BIT_AND => Ok(bin_op_sigma_parse(ArithOp::BitAnd.into(), r)?),
-                OpCode::BIT_XOR => Ok(bin_op_sigma_parse(ArithOp::BitXor.into(), r)?),
+                OpCode::BIT_OR => Ok(bin_op_sigma_parse(BitOp::BitOr.into(), r)?),
+                OpCode::BIT_AND => Ok(bin_op_sigma_parse(BitOp::BitAnd.into(), r)?),
+                OpCode::BIT_XOR => Ok(bin_op_sigma_parse(BitOp::BitXor.into(), r)?),
                 OpCode::BLOCK_VALUE => Ok(Expr::BlockValue(BlockValue::sigma_parse(r)?)),
                 OpCode::FUNC_VALUE => Ok(Expr::FuncValue(FuncValue::sigma_parse(r)?)),
                 OpCode::APPLY => Ok(Expr::Apply(Apply::sigma_parse(r)?)),
@@ -137,6 +153,7 @@ impl Expr {
                 CalcSha256::OP_CODE => Ok(CalcSha256::sigma_parse(r)?.into()),
                 And::OP_CODE => Ok(And::sigma_parse(r)?.into()),
                 Or::OP_CODE => Ok(Or::sigma_parse(r)?.into()),
+                Xor::OP_CODE => Ok(Xor::sigma_parse(r)?.into()),
                 Atleast::OP_CODE => Ok(Atleast::sigma_parse(r)?.into()),
                 OpCode::COLL => Ok(coll_sigma_parse(r)?.into()),
                 OpCode::COLL_OF_BOOL_CONST => Ok(bool_const_coll_sigma_parse(r)?.into()),
@@ -146,14 +163,17 @@ impl Expr {
                 ForAll::OP_CODE => Ok(ForAll::sigma_parse(r)?.into()),
                 BoolToSigmaProp::OP_CODE => Ok(BoolToSigmaProp::sigma_parse(r)?.into()),
                 Upcast::OP_CODE => Ok(Upcast::sigma_parse(r)?.into()),
+                Downcast::OP_CODE => Ok(Downcast::sigma_parse(r)?.into()),
                 If::OP_CODE => Ok(If::sigma_parse(r)?.into()),
                 ByIndex::OP_CODE => Ok(ByIndex::sigma_parse(r)?.into()),
                 SizeOf::OP_CODE => Ok(SizeOf::sigma_parse(r)?.into()),
+                Slice::OP_CODE => Ok(Slice::sigma_parse(r)?.into()),
                 CreateProveDlog::OP_CODE => Ok(CreateProveDlog::sigma_parse(r)?.into()),
                 CreateProveDhTuple::OP_CODE => Ok(CreateProveDhTuple::sigma_parse(r)?.into()),
                 SigmaPropBytes::OP_CODE => Ok(SigmaPropBytes::sigma_parse(r)?.into()),
                 Tuple::OP_CODE => Ok(Tuple::sigma_parse(r)?.into()),
                 DecodePoint::OP_CODE => Ok(DecodePoint::sigma_parse(r)?.into()),
+                SubstConstants::OP_CODE => Ok(SubstConstants::sigma_parse(r)?.into()),
                 ByteArrayToLong::OP_CODE => Ok(ByteArrayToLong::sigma_parse(r)?.into()),
                 ByteArrayToBigInt::OP_CODE => Ok(ByteArrayToBigInt::sigma_parse(r)?.into()),
                 LongToByteArray::OP_CODE => Ok(LongToByteArray::sigma_parse(r)?.into()),
@@ -162,7 +182,10 @@ impl Expr {
                 GetVar::OP_CODE => Ok(GetVar::sigma_parse(r)?.into()),
                 DeserializeRegister::OP_CODE => Ok(DeserializeRegister::sigma_parse(r)?.into()),
                 DeserializeContext::OP_CODE => Ok(DeserializeContext::sigma_parse(r)?.into()),
-                o => Err(SerializationError::NotImplementedOpCode(format!(
+                MultiplyGroup::OP_CODE => Ok(MultiplyGroup::sigma_parse(r)?.into()),
+                Exponentiate::OP_CODE => Ok(Exponentiate::sigma_parse(r)?.into()),
+                XorOf::OP_CODE => Ok(XorOf::sigma_parse(r)?.into()),
+                o => Err(SigmaParsingError::NotImplementedOpCode(format!(
                     "{0}(shift {1})",
                     o.value(),
                     o.shift()
@@ -173,8 +196,17 @@ impl Expr {
     }
 }
 
+trait SigmaSerializableWithOpCode: SigmaSerializable + HasOpCode {
+    fn sigma_serialize_w_opcode<W: SigmaByteWrite>(&self, w: &mut W) -> SigmaSerializeResult {
+        self.op_code().sigma_serialize(w)?;
+        self.sigma_serialize(w)
+    }
+}
+
+impl<T: SigmaSerializable + HasOpCode> SigmaSerializableWithOpCode for T {}
+
 impl SigmaSerializable for Expr {
-    fn sigma_serialize<W: SigmaByteWrite>(&self, w: &mut W) -> Result<(), io::Error> {
+    fn sigma_serialize<W: SigmaByteWrite>(&self, w: &mut W) -> SigmaSerializeResult {
         match self {
             Expr::Const(c) => match w.constant_store_mut_ref() {
                 Some(cs) => {
@@ -184,69 +216,79 @@ impl SigmaSerializable for Expr {
                 }
                 None => c.sigma_serialize(w),
             },
-            expr => {
-                let op_code = self.op_code();
-                op_code.sigma_serialize(w)?;
-                match expr {
-                    Expr::Const(_) => panic!("unexpected constant"), // handled in the code above (external match)
-                    Expr::Fold(op) => op.sigma_serialize(w),
-                    Expr::ConstPlaceholder(cp) => cp.sigma_serialize(w),
-                    Expr::ByteArrayToLong(s) => s.sigma_serialize(w),
-                    Expr::ByteArrayToBigInt(s) => s.sigma_serialize(w),
-                    Expr::LongToByteArray(s) => s.sigma_serialize(w),
-                    Expr::GlobalVars(_) => Ok(()),
-                    Expr::MethodCall(mc) => mc.sigma_serialize(w),
-                    Expr::ProperyCall(pc) => pc.sigma_serialize(w),
-                    Expr::Global => Ok(()),
-                    Expr::Context => Ok(()),
-                    Expr::OptionGet(v) => v.sigma_serialize(w),
-                    Expr::ExtractRegisterAs(v) => v.sigma_serialize(w),
-                    Expr::BinOp(op) => bin_op_sigma_serialize(op, w),
-                    Expr::BlockValue(op) => op.sigma_serialize(w),
-                    Expr::ValUse(op) => op.sigma_serialize(w),
-                    Expr::ValDef(op) => op.sigma_serialize(w),
-                    Expr::FuncValue(op) => op.sigma_serialize(w),
-                    Expr::Apply(op) => op.sigma_serialize(w),
-                    Expr::ExtractAmount(op) => op.sigma_serialize(w),
-                    Expr::SelectField(op) => op.sigma_serialize(w),
-                    Expr::CalcBlake2b256(op) => op.sigma_serialize(w),
-                    Expr::CalcSha256(op) => op.sigma_serialize(w),
-                    Expr::Collection(op) => coll_sigma_serialize(op, w),
-                    Expr::And(op) => op.sigma_serialize(w),
-                    Expr::Or(op) => op.sigma_serialize(w),
-                    Expr::Atleast(op) => op.sigma_serialize(w),
-                    Expr::LogicalNot(op) => op.sigma_serialize(w),
-                    Expr::Map(op) => op.sigma_serialize(w),
-                    Expr::Filter(op) => op.sigma_serialize(w),
-                    Expr::BoolToSigmaProp(op) => op.sigma_serialize(w),
-                    Expr::Upcast(op) => op.sigma_serialize(w),
-                    Expr::If(op) => op.sigma_serialize(w),
-                    Expr::ByIndex(op) => op.sigma_serialize(w),
-                    Expr::ExtractScriptBytes(op) => op.sigma_serialize(w),
-                    Expr::SizeOf(op) => op.sigma_serialize(w),
-                    Expr::CreateProveDlog(op) => op.sigma_serialize(w),
-                    Expr::CreateProveDhTuple(op) => op.sigma_serialize(w),
-                    Expr::ExtractCreationInfo(op) => op.sigma_serialize(w),
-                    Expr::Exists(op) => op.sigma_serialize(w),
-                    Expr::ExtractId(op) => op.sigma_serialize(w),
-                    Expr::SigmaPropBytes(op) => op.sigma_serialize(w),
-                    Expr::OptionIsDefined(op) => op.sigma_serialize(w),
-                    Expr::OptionGetOrElse(op) => op.sigma_serialize(w),
-                    Expr::Negation(op) => op.sigma_serialize(w),
-                    Expr::ForAll(op) => op.sigma_serialize(w),
-                    Expr::Tuple(op) => op.sigma_serialize(w),
-                    Expr::DecodePoint(op) => op.sigma_serialize(w),
-                    Expr::SigmaAnd(op) => op.sigma_serialize(w),
-                    Expr::SigmaOr(op) => op.sigma_serialize(w),
-                    Expr::GetVar(op) => op.sigma_serialize(w),
-                    Expr::DeserializeRegister(op) => op.sigma_serialize(w),
-                    Expr::DeserializeContext(op) => op.sigma_serialize(w),
-                }
+            Expr::Append(ap) => ap.sigma_serialize_w_opcode(w),
+            Expr::Fold(op) => op.sigma_serialize_w_opcode(w),
+            Expr::ConstPlaceholder(cp) => cp.sigma_serialize_w_opcode(w),
+            Expr::SubstConstants(s) => s.sigma_serialize_w_opcode(w),
+            Expr::ByteArrayToLong(s) => s.sigma_serialize_w_opcode(w),
+            Expr::ByteArrayToBigInt(s) => s.sigma_serialize_w_opcode(w),
+            Expr::LongToByteArray(s) => s.sigma_serialize_w_opcode(w),
+            Expr::GlobalVars(op) => op.op_code().sigma_serialize(w),
+            Expr::MethodCall(mc) => mc.sigma_serialize_w_opcode(w),
+            Expr::ProperyCall(pc) => pc.sigma_serialize_w_opcode(w),
+            Expr::Global => OpCode::GLOBAL.sigma_serialize(w),
+            Expr::Context => OpCode::CONTEXT.sigma_serialize(w),
+            Expr::OptionGet(v) => v.sigma_serialize_w_opcode(w),
+            Expr::ExtractRegisterAs(v) => v.sigma_serialize_w_opcode(w),
+            Expr::BinOp(op) => {
+                op.op_code().sigma_serialize(w)?;
+                bin_op_sigma_serialize(op, w)
             }
+            Expr::BlockValue(op) => op.sigma_serialize_w_opcode(w),
+            Expr::ValUse(op) => op.sigma_serialize_w_opcode(w),
+            Expr::ValDef(op) => op.sigma_serialize_w_opcode(w),
+            Expr::FuncValue(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Apply(op) => op.sigma_serialize_w_opcode(w),
+            Expr::ExtractAmount(op) => op.sigma_serialize_w_opcode(w),
+            Expr::SelectField(op) => op.sigma_serialize_w_opcode(w),
+            Expr::CalcBlake2b256(op) => op.sigma_serialize_w_opcode(w),
+            Expr::CalcSha256(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Collection(op) => {
+                op.op_code().sigma_serialize(w)?;
+                coll_sigma_serialize(op, w)
+            }
+            Expr::And(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Or(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Xor(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Atleast(op) => op.sigma_serialize_w_opcode(w),
+            Expr::LogicalNot(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Map(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Filter(op) => op.sigma_serialize_w_opcode(w),
+            Expr::BoolToSigmaProp(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Upcast(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Downcast(op) => op.sigma_serialize_w_opcode(w),
+            Expr::If(op) => op.sigma_serialize_w_opcode(w),
+            Expr::ByIndex(op) => op.sigma_serialize_w_opcode(w),
+            Expr::ExtractScriptBytes(op) => op.sigma_serialize_w_opcode(w),
+            Expr::SizeOf(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Slice(op) => op.sigma_serialize_w_opcode(w),
+            Expr::CreateProveDlog(op) => op.sigma_serialize_w_opcode(w),
+            Expr::CreateProveDhTuple(op) => op.sigma_serialize_w_opcode(w),
+            Expr::ExtractCreationInfo(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Exists(op) => op.sigma_serialize_w_opcode(w),
+            Expr::ExtractId(op) => op.sigma_serialize_w_opcode(w),
+            Expr::SigmaPropBytes(op) => op.sigma_serialize_w_opcode(w),
+            Expr::OptionIsDefined(op) => op.sigma_serialize_w_opcode(w),
+            Expr::OptionGetOrElse(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Negation(op) => op.sigma_serialize_w_opcode(w),
+            Expr::BitInversion(op) => op.sigma_serialize_w_opcode(w),
+            Expr::ForAll(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Tuple(op) => op.sigma_serialize_w_opcode(w),
+            Expr::DecodePoint(op) => op.sigma_serialize_w_opcode(w),
+            Expr::SigmaAnd(op) => op.sigma_serialize_w_opcode(w),
+            Expr::SigmaOr(op) => op.sigma_serialize_w_opcode(w),
+            Expr::GetVar(op) => op.sigma_serialize_w_opcode(w),
+            Expr::DeserializeRegister(op) => op.sigma_serialize_w_opcode(w),
+            Expr::DeserializeContext(op) => op.sigma_serialize_w_opcode(w),
+            Expr::MultiplyGroup(op) => op.sigma_serialize_w_opcode(w),
+            Expr::Exponentiate(op) => op.sigma_serialize_w_opcode(w),
+            Expr::XorOf(op) => op.sigma_serialize_w_opcode(w),
+            Expr::ExtractBytes(op) => op.sigma_serialize_w_opcode(w),
+            Expr::ExtractBytesWithNoRef(op) => op.sigma_serialize_w_opcode(w),
         }
     }
 
-    fn sigma_parse<R: SigmaByteRead>(r: &mut R) -> Result<Self, SerializationError> {
+    fn sigma_parse<R: SigmaByteRead>(r: &mut R) -> Result<Self, SigmaParsingError> {
         let tag = r.get_u8()?;
         Self::parse_with_tag(r, tag)
     }
@@ -255,14 +297,21 @@ impl SigmaSerializable for Expr {
 #[cfg(test)]
 #[cfg(feature = "arbitrary")]
 #[allow(clippy::unwrap_used)]
+#[allow(clippy::panic)]
 mod tests {
-    use crate::address::AddressEncoder;
-    use crate::address::NetworkPrefix;
+    use crate::chain::address::AddressEncoder;
+    use crate::chain::address::NetworkPrefix;
 
     use super::*;
     use crate::serialization::sigma_serialize_roundtrip;
     use proptest::prelude::*;
 
+    #[test]
+    fn ser_global() {
+        let e = Expr::Global;
+        assert_eq!(sigma_serialize_roundtrip(&e), e);
+    }
+
     proptest! {
 
         #[test]
