@@ -4,32 +4,22 @@ use super::sfunc::SFunc;
 use super::smethod::MethodId;
 use super::smethod::SMethodDesc;
 use super::stype::SType;
-use super::stype_companion::STypeCompanion;
-use super::stype_companion::STypeCompanionHead;
 use lazy_static::lazy_static;
 
-/// SGlobal type id
-pub const TYPE_ID: TypeCode = TypeCode::SGLOBAL;
+/// SGlobal type code
+pub const TYPE_CODE: TypeCode = TypeCode::SGLOBAL;
+/// SGlobal type name
+pub static TYPE_NAME: &str = "Global";
 
 /// groupGenerator property
 pub const GROUP_GENERATOR_METHOD_ID: MethodId = MethodId(1);
 /// "xor" predefined function
 pub const XOR_METHOD_ID: MethodId = MethodId(2);
 
-static S_GLOBAL_TYPE_COMPANION_HEAD: STypeCompanionHead = STypeCompanionHead {
-    type_id: TYPE_ID,
-    type_name: "Global",
-};
-
 lazy_static! {
-    /// Box object type companion
-    pub static ref S_GLOBAL_TYPE_COMPANION: STypeCompanion = STypeCompanion::new(
-        &S_GLOBAL_TYPE_COMPANION_HEAD,
-        vec![
-            &GROUP_GENERATOR_METHOD_DESC,
-            &XOR_METHOD_DESC,
-        ]
-    );
+    /// Global method descriptors
+    pub(crate) static ref METHOD_DESC: Vec<&'static SMethodDesc> =
+        vec![&GROUP_GENERATOR_METHOD_DESC, &XOR_METHOD_DESC,];
 }
 
 lazy_static! {
